import numpy as np
import pandas as pd
import os


def get_AM1p5_spectrum():
    """
    Get the AM1.5 spectrum. Data file available at:

    https://www.nrel.gov/grid/solar-resource/spectra-am1.5.html

    Returns
    -------

    df : dataframe

        AM1.5 spectral data as a dataframe.

    """
    cd = os.path.dirname(os.path.abspath(__file__))

    df = pd.read_csv(os.path.join(cd, 'astmg173.csv'), skiprows=1)
    return df


def solar_weighted_photon_reflectance(wavelength,
                                     reflectance,
                                     wavelength_min=400,
                                     wavelength_max=1100):
    """
    Calculate the solar-weighted photon reflectance (SWPR). This is equal to
    the reflectance weighted by the photon flux. Note this function can be
    also be used to calculate solar-weighted photon transmittance.

    Parameters
    ----------
    wavelength : ndarray

        wavelength values in nm

    reflection : ndarray

        reflectance (or transmittance) at the wavelength values.

    wavelength_min : float

        Minimum wavelength limit to the integral.

    wavelength_max : float

        Maximum wavelength limit to the integral.

    Returns
    -------

    swpr : float

        solar weighted photon reflectance.

    """
    cd = os.path.dirname(os.path.abspath(__file__))

    sun = pd.read_csv(os.path.join(cd, 'astmg173.csv'), skiprows=1)

    cax = np.logical_and(wavelength > wavelength_min,
                         wavelength < wavelength_max)
    wavelength = wavelength[cax]
    reflectance = reflectance[cax]
    # wavelength = np.linspace(200,1100,20)

    AM1p5 = np.interp(wavelength, sun['wavelength nm'],
                      sun['Global tilt  W*m-2*nm-1'])
    dwavelength = np.diff(wavelength)
    dwavelength = np.append(dwavelength, dwavelength[-1])

    photon_energy = 1 / wavelength
    swpr = np.sum(dwavelength * AM1p5 / photon_energy * reflectance) / \
          np.sum(dwavelength * AM1p5 / photon_energy)
    return swpr




"""

This module can be used to compare the output of the tmm package to the
corresponding optimized functions.

"""

import numpy as np
import pandas as pd
from tmm import coh_tmm, unpolarized_RT
from time import time
from numpy import pi, inf, cos, sin, exp



def thick_slab_reflectance_tmm(polarization, index_substrate, aoi, wavelength):
    """
    Reflection from a thick slab of material.

    Parameters
    ----------
    polarization : str

        Light polarization, can be "s" or "p" or "mixed"

    index_substrate : (N,) ndarray

        Index of refraction of the slab of material evaluated at the
        wavelengths specified in the `wavelength` input.

    aoi : float

        Angle of incidence in degrees

    wavelength : (N,) ndarray

        wavelength in nm, must be the same length as index_substrate.

    Returns
    -------

    """
    wavelength = wavelength.astype('float')
    degree = pi / 180
    R = np.zeros_like(wavelength)
    if polarization in ['s', 'p']:
        for j in range(len(R)):
            R[j] = coh_tmm(polarization,
                           [1.0003, index_substrate[j]],
                           [inf, inf],
                           aoi * degree, wavelength[j])['R']
    elif polarization == 'mixed':
        for j in range(len(R)):
            R[j] = unpolarized_RT([1.0003, index_substrate[j]], [inf, inf],
                                  aoi * degree, wavelength[j])['R']
    else:
        raise Exception("polarization must be 's','p' or 'mixed'")
    return R


def thin_film_reflectance_tmm(polarization, index_film, index_substrate,
                             film_thickness, aoi,
                             wavelength,
                             vectorize=False):
    """
    Calculate reflection from a thin film on a substrate. dimensions in nm.


    Parameters
    ----------
    polarization
    index_film
    index_substrate
    film_thickness
    aoi
    wavelength
    vectorize

    Returns
    -------

    """
    degree = pi / 180
    wavelength = wavelength.astype('float')

    d_list = [np.inf, film_thickness, np.inf]

    index_air = 1.0003

    R = np.zeros_like(wavelength)
    if polarization in ['s', 'p']:
        for j in range(len(R)):
            n_list = [1.0003, index_film[j], index_substrate[j]]
            R[j] = coh_tmm(polarization,
                           n_list,
                           d_list, aoi * degree, wavelength[j])['R']
    elif polarization == 'mixed':

        if vectorize:

            def unpolarized_RT_func(index_film, index_substrate, aoi,
                                    film_thickness,
                                    wavelength, ):
                return unpolarized_RT(
                    n_list=[index_air, index_film, index_substrate],
                    d_list=[np.inf, film_thickness, np.inf],
                    th_0=aoi * degree,
                    lam_vac=wavelength)['R']

            unpolarized_RT_vectorized = np.vectorize(unpolarized_RT_func)
            R = unpolarized_RT_vectorized(index_film, index_substrate, aoi,
                                          film_thickness, wavelength)
            print('vectorized done')
        else:

            for j in range(len(R)):
                n_list = [index_air, index_film[j], index_substrate[j]]
                R[j] = unpolarized_RT(n_list,
                                      d_list,
                                      aoi * degree,
                                      wavelength[j])['R']

    else:
        raise Exception("polarization must be 's','p' or 'mixed'")
    return R



import numpy as np


def refractive_index_glass(wavelength, type='soda-lime-low-iron'):
    """
    Return real part of refractive index for glass given an array of
    wavelengths.

    Data for Soda-lime glass from:
     https://refractiveindex.info/?shelf=glass&book=soda-lime&page=Rubin-clear

    Rubin 1985. Range of validity is 310-4600 nm.

    Data for BK7 glass from:
        https://refractiveindex.info/?shelf=glass&book=BK7&page=SCHOTT

    Parameters
    ----------
    wavelength : ndarray
        wavelength in nm

    type : str
        Type of glass. Options are:

        'soda-lime-low-iron'

    Returns
    -------
    index_of_refraction : ndarray
        Index of refraction at specified wavelength.

    """

    if type.lower()== 'soda-lime-low-iron':
        wavelength = wavelength / 1000
        n = 1.5130 - 0.003169 * wavelength ** 2 + 0.003962 * wavelength ** -2
    elif type.upper()=='BK7':
        wavelength = wavelength / 1000
        n = np.sqrt(1 + \
                    (1.03961212 * wavelength ** 2) / (
                            wavelength ** 2 - 0.00600069867) + \
                    (0.231792344 * wavelength ** 2) / (
                            wavelength ** 2 - 0.0200179144) + \
                    (1.01046945 * wavelength ** 2) / (
                                wavelength ** 2 - 103.560653)
                    )


    return n


def refractive_index_porous_silica(wavelength, porosity=0.5):
    """
    Calculates index of refraction for porous silica using the effective
    medium approximation and volume averaging theory.

    https://refractiveindex.info/?shelf=main&book=SiO2&page=Malitson

    Parameters
    ----------
    wavelength : numeric

        Wavelength in nm.

    porosity : float

        Fractional porosity, a number from 0 to 1.0.


    Returns
    -------

    index : numeric

        Refractive index

    """
    wavelength = wavelength / 1000
    n = np.sqrt(1 + \
                (0.6961663 * wavelength ** 2) / (
                        wavelength ** 2 - 0.06840432 ** 2) + \
                (0.4079426 * wavelength ** 2) / (
                        wavelength ** 2 - 0.11624142 ** 2) + \
                (0.8974794 * wavelength ** 2) / (
                        wavelength ** 2 - 9.8961612 ** 2)
                )
    n_air = 1.00029

    n_total = np.sqrt(n ** 2 * (1 - porosity) + n_air ** 2 * (porosity))

    return n_total



import numpy as np
import pandas as pd

def read_oceanview_file(filename):
    """
    Read spectral file saved using Ocean Insight oceanview spectrometer
    software.

    Parameters
    ----------
    filename : str

        filename to import

    Returns
    -------

    data : dict

        Dictionary containing data from file. Includes fields:

        filename: name of file.
        wavelength: wavlength in nm
        value: Spectral data, can be reflectance, intensity, etc.
        Number of Pixels in Spectrum: length of spectrum.
        Integration Time (sec): integrationg time in seconds.
        Scans to average: number of scans to average.

    """
    data = {}
    data['filename'] = filename

    # Read header.
    with open(filename, 'r') as file:

        # is_header = True
        for j in range(100):

            line = file.readline()

            if line[:5] == '>>>>>':
                break

            if line.find(':') >= 0:
                line_parts = line.split(': ')
                data[line_parts[0]] = line_parts[1].replace('\n', '')

    skip_rows = j + 1

    data['Number of Pixels in Spectrum'] = int(data['Number of Pixels in Spectrum'])
    data['Integration Time (sec)'] = float(data['Integration Time (sec)'])
    data['Scans to average'] = int(data['Scans to average'])

    df = pd.read_csv(filename,
                     delimiter='\t',
                     skiprows=skip_rows,
                     header=None,
                     names=['wavelength', 'value']
                     )

    # Sometimes there are repeated pixels.
    data['wavelength'] = np.array(df['wavelength'])[
                         :data['Number of Pixels in Spectrum']]
    data['value'] = np.array(df['value'])[
                    :data['Number of Pixels in Spectrum']]
    return data


def read_oceanview_image_set(filenames):
    """
    Import a set of files using read_oceanview_file

    Parameters
    ----------
    filenames : list

        list of filenames

    Returns
    -------

    data : dict

        Data dictionary similar to the function read_oceanview_file, except
        an additinoal field:

        value_mean: mean of the 'value' at each wavelength.

    """
    if len(filenames) == 0:
        raise Exception('no filenames given.')

    for k in range(len(filenames)):
        ref = read_oceanview_file(filenames[k])
        if k == 0:
            ref0 = ref.copy()
            value = np.zeros(
                (len(filenames), len(ref['wavelength'])))
            wavelength = ref['wavelength']

        value[k, :] = ref['value']

    value_mean = np.mean(value, axis=0)

    ref0['value'] = value
    ref0['value_mean'] = value_mean

    return ref0